//***************************************************************************
//*  ====================
//*  AZ2JAnalysis Classes
//*  ====================
//*
//*  (Description)
//*	A user analysis class for ILC analyses.
//*	This reads and analyzes MC e+e- -> AZ data.
//*  (Requires)
//*	library Anlib (in physsim-99a-1 if K.Fujii)
//*	library AZStudy (also in physsim)
//*  (Provides)
//*	class AZ2JAnalysis
//*	class AZ2JAnalysisBuf
//*  (Usage)
//*	...
//*  (Update Record)
//*     2019/12/11	K.Fujii		Original version.
//***************************************************************************

#include "AZ2JAnalysis.h"
#include "AZSpring.h"
#include "TROOT.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH1D.h"
#include "JSFSteer.h"
#include "JSFSIMDST.h"
#include "Anlib.h"
#include "ANLTrack.h"

#include <sstream>
#include <iomanip>

static const Double_t kMassZ   = 91.19;	// Z mass
static const Double_t kSigmaMz =   4.0;	// Z mass resolution

typedef enum { kElectron = 11, kMuon = 13 } EPID;

Bool_t gDEBUG = kFALSE;
Int_t  gNgoods  = 0;
const Int_t  kMaxCuts = 100;
      stringstream gCutName[kMaxCuts];
      TH1D  *hStat = 0;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ---------------------
//  AZ2JAnalysisBuf Class
//  ---------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ClassImp(AZ2JAnalysisBuf)

// Constructors
AZ2JAnalysisBuf::AZ2JAnalysisBuf(const Char_t *name, 
		                 const Char_t *title,
                                 AZ2JAnalysis *module) 
	: JSFEventBuf(name, title, (JSFModule*)module)
{
}

AZ2JAnalysisBuf::AZ2JAnalysisBuf(AZ2JAnalysis *module, 
		                 const Char_t *name,
                                 const Char_t *title) 
	: JSFEventBuf(name, title, (JSFModule*)module) 
{
}

AZ2JAnalysisBuf::~AZ2JAnalysisBuf() 
{
}


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ------------------
//  AZ2JAnalysis Class
//  ------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ClassImp(AZ2JAnalysis)

// Constructor
AZ2JAnalysis::AZ2JAnalysis(const Char_t *name, 
		           const Char_t *title)
             : JSFModule(name, title),
               fNtracksCut(    5),   // No. of tracks
               fEtrackCut (  0.05),  // track energy
               fEvisCut   ( 50.0),   // Minimum visible energy
               fPtCut     (  0.0),   // Pt minimum
               fPlCut     (999.0),   // Pl maximum
               fEphotonCut( 50.0),   // minimum photon energy
               fCosConeCut(  0.94),  // |cos(theta_cone)|
               fEconeCut  (  2.0),   // Econe maximum
               fYcutCut   (0.004),   // y_cut to force the event to 4 jets
               fNjetsCut  (    2),   // No. of jets
               fEjetCut   (  5.0),   // E_jet minimum
               fCosjetCut (999.0),   // |cos(theta_jet)| maximum
               fCos2jCut  (999.0),   // |cos(theta_qq)| maximum
               fM2jCut    ( 18.0),   // |m_qq-m_H| maximum
               fNpinjCut  (    3)    // |m_qq-m_H| maximum
{
  fEventBuf = new AZ2JAnalysisBuf(this);
  SetBufferSize(2000);	// buffer size for event data
  cout << "AZ2JAnalysis is created... fEventBuf is "
       << (void *)fEventBuf << endl;
  //--
  //  Read in Generator info.
  //--
  AZBases *bsp = static_cast<AZBases *>(gROOT->FindObject("AZBases"));
  if (bsp) {
    cerr << "------------------------------------" << endl
         << " Ecm = " << bsp->GetEcmInit() << " GeV" << endl
         << "------------------------------------" << endl;
    SetEcm(bsp->GetEcmInit()); 
  }
}

// Destructor
AZ2JAnalysis::~AZ2JAnalysis()
{
  cout << "AZ2JAnalysisBuf will be deleted... fEventBuf is "
       << (void *)fEventBuf << endl;
  delete fEventBuf; fEventBuf = 0;
}

//####### Functions ###########
//
// ******Initialize()****** //
Bool_t AZ2JAnalysis::Initialize()
{
  return kTRUE;
}
// *****End of Initialize()***** //

// *****Process()***** //
Bool_t AZ2JAnalysis::Process(Int_t ev)
{
  //--
  // Remember the previous directory.
  //--
  TDirectory *last = gDirectory;
  gFile->cd("/");
  if (!hStat) hStat = new TH1D("hStat","Cut Statistics", 20, 0., 20.);

  static TNtupleD *hEvt = 0;
  if (!hEvt) {
    stringstream tupstr;
    tupstr << "nev:ecm:ntracks:evis:pt:pl:mqq:ej1:ej2:cosj1:cosj2"
	   << ":mj1:mj2:cosz:npj1:npj2"
	   << ends;
    hEvt = new TNtupleD("hEvt", "", tupstr.str().data());
  }

  // ---------------------
  // Analysis starts here.
  // ---------------------
  Float_t selid = -0.5;
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "No Cuts" << ends;
  }

  //--
  // Get event buffer and make combined tracks accessible.
  //--
  JSFSIMDST       *sdsp = static_cast<JSFSIMDST *>(gJSF->FindModule("JSFSIMDST"));
  JSFSIMDSTBuf    *evtp = static_cast<JSFSIMDSTBuf *>(sdsp->EventBuf());
  AZ2JAnalysisBuf *uap  = static_cast<AZ2JAnalysisBuf*>(fEventBuf);
  AZ2JAnalysisBuf &a	= *uap;

  Int_t		 ntrks	= evtp->GetNLTKCLTracks();	// No. of tracks
  TObjArray	*trks	= evtp->GetLTKCLTracks();	// Combined tracks

  //--
  // Select good tracks.
  //--
  ANL4DVector qsum;
  TObjArray tracks(1000);
  tracks.SetOwner();
  Int_t ntracks = 0;
  for ( Int_t i = 0; i < ntrks; i++) {
    JSFLTKCLTrack *tp = static_cast<JSFLTKCLTrack*>(trks->UncheckedAt(i));
    if ( tp->GetE() > fEtrackCut ) {
      ANLTrack *qtp = new ANLTrack(tp);
      tracks.Add(qtp);		// track 4-momentum
      qsum += *qtp;		// total 4-mometum
      ntracks++;		// *qt stays.
    }
  }
  if (gDEBUG) cerr << "Ntracks = " << ntracks << endl;

  //--
  // Cut on No. of tracks.
  //--
  if (ntracks < fNtracksCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Ntracks >= " << fNtracksCut << ends;
  }
  
  Double_t evis = qsum(0);	// E_vis
  Double_t pt	= qsum.GetPt(); // P_t
  Double_t pl	= qsum(3);	// P_l

  if (gDEBUG) cerr << "Evis = " << evis 
	           << " Pt = " << pt << " Pl = " << pl << endl;

  //--
  // Cut on Evis.
  //--
  if (evis < fEvisCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Evis >= " << fEvisCut << ends;
  }

  //--
  // Cut on Pt.
  //--
  if (pt > fPtCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Pt <= " << fPtCut << ends;
  }

  //--
  // Cut on Pl.
  //--
  if (TMath::Abs(pl) > fPlCut) {return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "|Pl| <= " << fPlCut << ends;
  }

  //--
  // Find Isolated photon.
  //--
  TObjArray isophotons(20);
  Int_t fNisophotons = 0;
  TIter nexttrk (&tracks);
  ANLTrack *trkp;
  while ((trkp = static_cast<ANLTrack *>(nexttrk()))) {
    ANLTrack &trk = *trkp;
    if (!trk.IsGamma()) continue;
    Double_t ephoton = trk.E();
    if (ephoton < fEphotonCut) continue;
    Double_t econe = trk.GetConeEnergy(fCosConeCut, &tracks);
    if (econe <= fEconeCut) {
      isophotons.Add(trkp);
      fNisophotons++;
      trk.Lock();
    }
  }

  //--
  // Require one isolated photon.
  //--
  if (fNisophotons != 1) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Nphotons = 1" << ends;
  }

  //--
  // Find jets.
  //--
  ANLJadeEJetFinder jclust(fYcutCut);
  jclust.Initialize(tracks);
  jclust.FindJets();
  Double_t ycut  = jclust.GetYcut();
  Int_t    njets = jclust.GetNjets();
  
  if (gDEBUG) cerr << "Ycut = " << ycut << " Njets = " << njets << endl;

  //--
  // Cut on No. of jets.
  //--
  if (njets < fNjetsCut) { return kFALSE;}
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Njets >= " << fNjetsCut << " for Ycut = " << fYcutCut << ends;
  }

  //--
  // Now force the event to be fNjetsCut.
  //--
  jclust.ForceNJets(fNjetsCut);
  njets = jclust.GetNjets();
  ycut  = jclust.GetYcut();

  if(gDEBUG) cerr << "Ycut = " << ycut << " Njets = " << njets << endl;

  //--
  // Make sure that No. of jets is fNjetsCut.
  //--
  if (njets != fNjetsCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Njets = " << fNjetsCut << ends;
  }

  //--
  // Loop over jets and decide Ejet_min and |cos(theta_j)|_max.
  //--
  TObjArray &jets = jclust.GetJets();
  ANLJet &j1     = *static_cast<ANLJet *>(jets.At(0));
  Int_t    npj1  = j1.GetNparticles();
  Double_t ej1   = j1.E();
  Double_t mj1   = j1.GetMass();
  Double_t cosj1 = j1.CosTheta();
  ANLJet &j2     = *static_cast<ANLJet *>(jets.At(1));
  Int_t    npj2  = j2.GetNparticles();
  Double_t ej2   = j2.E();
  Double_t mj2   = j2.GetMass();
  Double_t cosj2 = j2.CosTheta();

  //--
  // Cut on Ejet_min.
  //--
  if (ej1 < fEjetCut || ej2 < fEjetCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "Ejet >= " << fEjetCut << ends;
  }

  //--
  // Cut on |cos(theta_j)|_max.
  //--
  if (TMath::Abs(cosj1) > fCosjetCut || TMath::Abs(cosj2) > fCosjetCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "|cos(theta_j)| <= " << fCosjetCut << ends;
  }

  //--
  // Unlock photon tracks.
  //--
  TIter nextphoton(&isophotons);
  while ((trkp = static_cast<ANLTrack *>(nextphoton()))) trkp->Unlock();
  ANL4DVector isophoton = *static_cast<ANL4DVector *>(isophotons.At(0));

  //--
  // Find Z candidates in the given mass window.
  //--
  ANL4DVector z = j1 + j2;
  Double_t zmass = z.GetMass();
  if (TMath::Abs(zmass - kMassZ) > fM2jCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "|m_qq - m_z| <= " << fM2jCut << ends;
  }

  //--
  // Cut on cos(theta_z).
  //--
  Double_t cosz = z.CosTheta();
  if (TMath::Abs(cosz) > fCos2jCut) { return kFALSE; }
  hStat->Fill(++selid);
  if (gNgoods == 0) {
    gCutName[(Int_t)selid] << "|cos(theta_z)| <= " << fCos2jCut << ends;
  }

  // ----------------------
  // End of event selection
  // ----------------------
  gNgoods++;
  cerr << "--------------------------------------------------" << endl
       << "Event "                   << gJSF->GetEventNumber() << endl
       << "--------------------------------------------------" << endl;

  //--
  // Now store this in AZ2JAnalysisBuf.
  //--
  a.fNtracks	= ntracks;
  a.fEvis	= evis;
  a.fPt		= pt;
  a.fPl		= pl;
  a.fYcut	= ycut;
  a.fNjets	= njets;

  //--
  // Now fill ntuples.
  //--    
    Double_t data[100];
    data[ 0] = gJSF->GetEventNumber();
    data[ 1] = GetEcm();
    data[ 2] = ntracks;
    data[ 3] = evis;
    data[ 4] = pt;
    data[ 5] = pl;
    data[ 6] = zmass;
    data[ 7] = ej1;
    data[ 8] = ej2;
    data[ 9] = cosj1;
    data[10] = cosj2;
    data[11] = mj1;
    data[12] = mj2;
    data[13] = cosz;
    data[14] = npj1;
    data[15] = npj2;

    hEvt->Fill(data);

  // ------------------------
  //  That's it.
  // ------------------------
  last->cd();
  return kTRUE;
}
// *****End of Process()***** //


// *****Terminate()***** //
Bool_t AZ2JAnalysis::Terminate()
{
  // This function is called at the end of job.
  //--
  // Print out cut statistics
  //--
  cerr << endl
       << "  =============" << endl
       << "   Cut Summary " << endl
       << "  =============" << endl
       << endl
       << "  -----------------------------------------------------------" << endl
       << "   ID   No.Events    Cut Description                         " << endl
       << "  -----------------------------------------------------------" << endl;
  for (int id=0; id<kMaxCuts && gCutName[id].str().data()[0]; id++) {
    cerr << "  " << setw( 3) << id
         << "  " << setw(10) << static_cast<int>(hStat->GetBinContent(id+1))
         << "  : " << gCutName[id].str().data() << endl;
  }
  cerr << "  -----------------------------------------------------------" << endl;
 
  //--
  // That's it!
  //--
  return 0;
}
// *****End of Terminate()***** //
