#ifndef AZ2JANALYSIS_H
#define AZ2JANALYSIS_H

//**********************************************************************
//* ======================
//*  AZ2JAnalysis Classes
//* ======================
//*
//* (Description)
//*    Analysis class for e+e- -> AH -> 2 jets + gamma.
//* (Requires)
//*	library Anlib (from physsim of K. Fujii)
//*	library AZStudy (also in physsim libraries)
//* (Provides)
//*	class AZ2JAnalysis
//*	class AZ2JAnalysisBuf
//* (Usage)
//*	...
//* (Update Record)
//:   2019/12/11	K.Fujii		Original version.
//**********************************************************************

#include "JSFModule.h"
#include "TMath.h"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ---------------------
//  AZ2JAnalysisBuf Class
//  ---------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class AZ2JAnalysis;

class AZ2JAnalysisBuf : public JSFEventBuf{
friend class AZ2JAnalysis;
public:
  AZ2JAnalysisBuf(const char *name="AZ2JAnalysisBuf",
                  const char *title="AZ 2-Jet Data", AZ2JAnalysis *mod=0);
  AZ2JAnalysisBuf(AZ2JAnalysis *mod, const char *name="AZ2JAnalysisBuf",
                  const char *title="AZ 2-Jet Data");
  virtual ~AZ2JAnalysisBuf();

  inline Int_t    GetNtracks()	const {return fNtracks; }
  inline Double_t GetEvis()	const {return fEvis; }
  inline Double_t GetPt()	const {return fPt; }
  inline Double_t GetPl()	const {return fPl; }
  inline Int_t    GetNjets()	const {return fNjets; }
  inline Double_t GetYcut()	const {return fYcut; }

private:
  Int_t		fNtracks;	// track multiplicity
  Int_t         fNisophotons;   // isolated photon multiplicity
  Double_t	fEvis;		// visible energy (GeV)
  Double_t	fPt;		// Pt
  Double_t	fPl;		// Pl
  Double_t	fYcut;		// y_cut to force the event to 4 jets
  Int_t		fNjets;		// jet multiplicity

  ClassDef(AZ2JAnalysisBuf, 2) // AZ2JAnalysis Buffer Example
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ------------------
//  AZ2JAnalysis Class
//  ------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

class AZ2JAnalysis : public JSFModule {
public:
  AZ2JAnalysis() : JSFModule("AZ2JAnalysis","AZ2JAnalysis Example") {}
  AZ2JAnalysis(const Char_t *name, const Char_t *title);
  virtual ~AZ2JAnalysis();
  
  inline void SetEcm       (Double_t x) { fEcm        = x; }
  inline void SetNtrackCut (Int_t    x) { fNtracksCut = x; }
  inline void SetEtrackCut (Double_t x) { fEtrackCut = x; }
  inline void SetEvisCut   (Double_t x) { fEvisCut = x; }
  inline void SetPtCut     (Double_t x) { fPtCut = x; }
  inline void SetPlCut     (Double_t x) { fPlCut = x; }
  inline void SetEphotonCut(Double_t x) { fEphotonCut = x; }
  inline void SetConeAngle (Double_t x) 
                           { fCosConeCut = TMath::Cos(TMath::Pi()*x/180.); }
  inline void SetEconeCut  (Double_t x) { fEconeCut = x; }
  inline void SetMinYcut   (Double_t x) { fYcutCut = x; }
  inline void SetNjetCut   (Int_t    x) { fNjetsCut = x; }
  inline void SetEjetCut   (Double_t x) { fEjetCut = x; }
  inline void SetCosjetCut (Double_t x) { fCosjetCut = x; }
  inline void SetCoszCut   (Double_t x)	{ fCos2jCut = x; }
  inline void SetM2jCut    (Double_t x) { fM2jCut = x; }
  inline void SetNpinjCut  (Double_t x) { fNpinjCut= x; }

  inline Double_t GetEcm() const { return fEcm; }

  Bool_t Initialize();
  Bool_t Process(Int_t evt);
  Bool_t Terminate();

private:
  Double_t fEcm;        // Ecm
  Int_t    fNtracksCut;	// No. of tracks
  Double_t fEtrackCut;	// track energy
  Double_t fEvisCut;	// minimum visible energy
  Double_t fPtCut;	// Pt minimum
  Double_t fPlCut;	// Pl maximum
  Double_t fEphotonCut; // Ephoton mimimum
  Double_t fCosConeCut; // cos(theta_cone)
  Double_t fEconeCut;   // Ecome maximum
  Double_t fYcutCut;	// y_cut to force the event to 4 jets
  Int_t    fNjetsCut;	// No. of Jets
  Double_t fEjetCut;	// E_jet minimum
  Double_t fCosjetCut;	// |cos(theta_j)| maximum
  Double_t fCos2jCut;	// |cos(theta_z)| and |cos(theta_h)| maximum
  Double_t fM2jCut;	// |m_jj - m_w| maximum
  Double_t fNpinjCut;	// No. particles in jet

  ClassDef(AZ2JAnalysis, 2)	// AZ2JAnalysis Example
};
#endif
