#ifndef HCHCSPRING_H
#define HCHCSPRING_H
//*****************************************************************************
//* =====================
//*  HCHCSpring
//* =====================
//*  
//* (Description)
//*    2HDM: e+e- -> H+H- generator
//*
//* (Update Record)
//*    2017/01/19  K.Fujii	Original version.
//*
//*****************************************************************************

#include "TNamed.h"
#include "JSFModule.h"
#include "JSFBases.h"
#include "JSFSpring.h"
#include "JSFBeamGeneration.h"

#include "HELLib.h"
#include "GENLib.h"

class HCBoson;
class TopQuark;

//_______________________________________________________________________
// =====================
//  class HCHCBases
// =====================
//-----------------------------------------------------------------------
class HCHCBases : public JSFBases {
friend class HCHCSpringBuf;
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  HCHCBases(const char *name  = "HCHCBases", 
            const char *title = "HCHC Bases");
  virtual ~HCHCBases();

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetMass      ()           const { return fMass;       }
  Double_t GetTanBeta   ()           const { return fTanBeta;    }
  Double_t GetTopMass   ()           const { return fTopMass;    }
  Double_t GetEcmInit   ()           const { return fEcmInit;    }

  Double_t GetQ2HH      ()           const { return fQ2HH;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2HM      ()           const { return fQ2HM;       }
  Double_t GetCosThetaT1()           const { return fCosThetaT1; }
  Double_t GetPhiT1     ()           const { return fPhiT1;      }
  Double_t GetQ2T1      ()           const { return fQ2T1;       }
  Double_t GetCosThetaWM()           const { return fCosThetaWM; }
  Double_t GetPhiWM     ()           const { return fPhiWM;      }
  Double_t GetQ2WM      ()           const { return fQ2WM;       }
  Double_t GetCosThetaF1()           const { return fCosThetaF1; }
  Double_t GetPhiF1     ()           const { return fPhiF1;      }
  Double_t GetQ2HP      ()           const { return fQ2HP;       }
  Double_t GetCosThetaT2()           const { return fCosThetaT2; }
  Double_t GetPhiT2     ()           const { return fPhiT2;      }
  Double_t GetQ2T2      ()           const { return fQ2T2;       }
  Double_t GetCosThetaWP()           const { return fCosThetaWP; }
  Double_t GetPhiWP     ()           const { return fPhiWP;      }
  Double_t GetQ2WP      ()           const { return fQ2WP;       }
  Double_t GetCosThetaF2()           const { return fCosThetaF2; }
  Double_t GetPhiF2     ()           const { return fPhiF2;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  void     SetMass     (Double_t m      )  { fMass      = m;     }
  void     SetTanBeta  (Double_t b      )  { fTanBeta   = b;     }
  void     SetTopMass  (Double_t m      )  { fTopMass   = m;     }
  void     SetEcmInit  (Double_t ecm    )  { fEcmInit   = ecm;   }
  void     SetISR      (Bool_t b = kTRUE)  { fISR       = b;     }
  void     SetBeamStr  (Bool_t b = kTRUE)  { fBeamStr   = b;     }
  void     SetBeamWidth(Double_t w      )  { fBeamWidth = w;     }
  void     SetPole     (Double_t p      )  { fPole      = p;     }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual void     Userin();   // Bases user initialization
  virtual void     Userout();  // Bases user output 

  Double_t Func();     // Bases integration function.

  // ----------------------
  //   Utility methods
  // ----------------------
private:
  void     SelectHelicities(Double_t &weight);

  Double_t  DSigmaDX     (GENBranch &cmbranch);
  Double_t  AmpSquared   (GENBranch &cmbranch);
  Complex_t FullAmplitude();
  Complex_t AmpEEtoHH    (const HELFermion &em,
                          const HELFermion &ep,
                          const HELScalar  &xm,
                          const HELScalar  &xp);

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Job parameters
  // ----------------
  Double_t fMass;           // m_H- : mass of H-
  Double_t fTopMass;        // top mass
  Double_t fTanBeta;        // top mass

  Double_t fEcmInit;        // Initial Ecm
  Int_t    fISR;            // ISR on?
  Int_t    fBeamStr;        // Beamstrahlung on?
  Double_t fBeamWidth;      // Beam width relative to nominal Ebm
#ifdef WITH_DBD_STANDARD
  Int_t    fISRBM;         // tag for BM and ISR
  Int_t    fISR_LLA_Order; // ISR LLA order
  Int_t    fStoreRemnants; // Store remnants or not (0,1)=(no, yes)
  Int_t    fStoreBeams;    // Store e+e- beams after Beamstrahlung (0,1)=(no,yes)
  std::string  fLumiFileDirectory; // directory of lumi file.
#endif
  Double_t fPole;           // electron polarization
  Int_t    fWmModesLo;      // W- decay mode lo;
  Int_t    fWmModesHi;      // W- decay mode hi;
  Int_t    fWpModesLo;      // W+ decay mode lo;
  Int_t    fWpModesHi;      // W+ decay mode hi;

  // ----------------
  //  Particle Data
  // ----------------
  HCBoson      *fHCBosonPtr;      //! PD table entry of "X"
  GENPDTWBoson *fWMBosonPtr;      //! PD table entry of "W-"
  GENPDTWBoson *fWPBosonPtr;      //! PD table entry of "W+"
  GENPDTZBoson *fZBosonPtr;       //! PD table entry of "Z"
  GENPDTPhoton *fPhotonPtr;       //! PD table entry of "Gamma"
  GENPDTGluon  *fGluonPtr;        //! PD table entry of "Gluon"

  // ----------------
  //  Event info
  // ----------------
  Double_t       fZBoost;         // p_z(cm)      in lab frame
  Double_t       fEcmIP;          // Ecm after B-strahlung
  Double_t       fQ2HH;           // q^2 of HH system
  Double_t       fQ2HM;           // q^2 of final state HM
  Double_t       fQ2HP;           // q^2 of final state HP
  Double_t       fQ2T1;           // q^2 of final state tbar
  Double_t       fQ2T2;           // q^2 of final state t
  TopQuark      *fT1Ptr;          // pointer to tb from H-
  GENDecayMode  *fWMModePtr;      // pointer to W- decay mode
  GENPDTEntry   *f1Ptr;           // pointer to 1st W- daughter
  GENPDTEntry   *f2Ptr;           // pointer to 2nd W- daughter
  Double_t       fQ2WM;           // q^2 of final state W-
  TopQuark      *fT2Ptr;          // pointer to t  from H+
  GENDecayMode  *fWPModePtr;      // pointer to W+ decay mode
  GENPDTEntry   *f4Ptr;           // pointer to 1st W+ daughter
  GENPDTEntry   *f5Ptr;           // pointer to 2nd W+ daughter
  Double_t       fQ2WP;           // q^2 of final state W+
  Int_t          fHelInitial[2];  // initial state helicities
  Int_t          fHelFinal  [8];  // final   state helicities
  Int_t          fJCombI;         // intial helicity combination
  Int_t          fJCombF;         // final  helicity combination
  Int_t          fWMMode;         // W- decay mode
  Int_t          fWPMode;         // W+ decay mode
  ANL4DVector    fK[2];           // [0,1] = [e-, e+]
#ifdef WITH_DBD_STANDARD
  ANL4DVector    fQV;             // four momentum of HCHC system
  ANL4DVector    fP[10];          // [0,1,2,3,4,5,6,7] = [b , btb, fwm1, fwm2, bb,  bt, fwp1, fwp2, fISR1, fISR2]
#else
  ANL4DVector    fP[8];           // [0,1,2,3,4,5,6,7] = [b , btb, fwm1, fwm2, bb,  bt, fwp1, fwp2]
#endif
  Double_t       fM[8];           // [0,1,2,3,4,5,6,7] = [m3, m7 , m11 , m12 , m5,  m9, m13 , m14 ]

  // ----------------
  //  Integ. vars
  // ----------------
  Double_t       fHelCombInitial; // initial state helicity combination
  Double_t       fHelCombFinal;   // final   state helicity combination
  Double_t       fWMDecayMode;    // decay mode selector for W-
  Double_t       fWPDecayMode;    // decay mode selector for W+
  Double_t       fCosTheta;       // cos(theta_H-) in cm  frame
  Double_t       fPhi;            // phi_H-        in cm  frame
  Double_t       fCosThetaT1;     // cos(theta_tb) in H-  frame
  Double_t       fPhiT1;          // phi_tb        in H-  frame
  Double_t       fXQ2HM;          // q^2 of final state H-
  Double_t       fCosThetaWM;     // cos(theta_W-) in tb  frame
  Double_t       fPhiWM;          // phi_W-        in tb  frame
  Double_t       fXQ2T1;          // q^2 of final state tb
  Double_t       fCosThetaF1;     // cos(theta_f1) in W-  frame
  Double_t       fPhiF1;          // phi_f1        in W-  frame
  Double_t       fXQ2WM;          // q^2 of final state W-
  Double_t       fCosThetaT2;     // cos(theta_t)  in H+  frame
  Double_t       fPhiT2;          // phi_t         in H+  frame
  Double_t       fXQ2HP;          // q^2 of final state H+
  Double_t       fCosThetaWP;     // cos(theta_W+) in t   frame
  Double_t       fPhiWP;          // phi_W+        in t   frame
  Double_t       fXQ2T2;          // q^2 of final state t 
  Double_t       fCosThetaF2;     // cos(theta_f1) in W+  frame
  Double_t       fPhiF2;          // phi_f1        in W+  frame
  Double_t       fXQ2WP;          // q^2 of final state W+

  Double_t       fR_BW_m;         //! naturl Ebeam spread for e-
  Double_t       fR_BW_p;         //!                     for e+
  Double_t       fR_BS_m;         //! beamstrahlung       for e-
  Double_t       fR_BS_p;         //!                     for e+

  Double_t       fR_ISR_var;      //! initial state radiation
  Double_t       fR_ISR_side;     //! from which side ? (e-/e+) 

#ifdef WITH_DBD_STANDARD
  Double_t       fR_BM_m;         //! beamstrahlung       for e-
  Double_t       fR_BM_p;         //!                     for e+

  Double_t       fR_ISR_m;      //! initial state radiation e-
  Double_t       fR_ISR_p;     //! initial state radiation e+
  Double_t       fR_ISR_1;      //!Pt of 1st initial state radiation
  Double_t       fR_ISR_2;     //! Azimuthal angle of 1st initial state radiation
  Double_t       fR_ISR_3;      //!Pt of 2nd initial state radiation
  Double_t       fR_ISR_4;     //! Azimuthal angle of 2nd initial state radiation
#endif
  
  Double_t       fXU[14];         //! [0-13] = (cosH, phiH, costb, phitb, coswm, phiwm, cosf1, phif1, 
  Double_t       fXL[14];         //!                       cost , phit , coswp, phiwp, cosf2, phif2)

  TFile   *fBeamFile;         //! Beamstrahlung data file
  JSFBeamGenerationCain *fBM; //! Beamstrahlung data

  ClassDef(HCHCBases, 1) // Bases for e+e- -> HH process
};

class HCHCSpring;

//_______________________________________________________________________
// =====================
//  class HCHCSpringBuf
// =====================
//-----------------------------------------------------------------------
class HCHCSpringBuf : public JSFSpringBuf {
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  HCHCSpringBuf(const char   *name   = "HCHCSpringBuf", 
                const char   *title  = "HCHC Spring test event buffer",
	          HCHCSpring *spring = 0)
	     : JSFSpringBuf(name,title,(JSFSpring*)spring) {} 
  virtual ~HCHCSpringBuf() {}

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetQ2HH      ()           const { return fQ2HH;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2HM      ()           const { return fQ2HM;       }
  Double_t GetCosThetaT1()           const { return fCosThetaT1; }
  Double_t GetPhiT1     ()           const { return fPhiT1;      }
  Double_t GetQ2T1      ()           const { return fQ2T1;       }
  Double_t GetCosThetaWM()           const { return fCosThetaWM; }
  Double_t GetPhiWM     ()           const { return fPhiWM;      }
  Double_t GetQ2WM      ()           const { return fQ2WM;       }
  Double_t GetCosThetaF1()           const { return fCosThetaF1; }
  Double_t GetPhiF1     ()           const { return fPhiF1;      }
  Double_t GetQ2HP      ()           const { return fQ2HP;       }
  Double_t GetCosThetaT2()           const { return fCosThetaT2; }
  Double_t GetPhiT2     ()           const { return fPhiT2;      }
  Double_t GetQ2T2      ()           const { return fQ2T2;       }
  Double_t GetCosThetaWP()           const { return fCosThetaWP; }
  Double_t GetPhiWP     ()           const { return fPhiWP;      }
  Double_t GetQ2WP      ()           const { return fQ2WP;       }
  Double_t GetCosThetaF2()           const { return fCosThetaF2; }
  Double_t GetPhiF2     ()           const { return fPhiF2;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual Bool_t SetPartons();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Event info
  // ----------------
  Double_t fQ2HH;            // q^2 of HH system
  Double_t fCosTheta;        // cos(theta_H-) in cm  frame
  Double_t fPhi;             // phi_H-        in cm  frame
  Double_t fQ2HM;            // q^2 of final state H-
  Double_t fCosThetaT1;      // cos(theta_tb) in H-  frame
  Double_t fPhiT1;           // phi_tb        in H-  frame
  Double_t fQ2T1;            // q^2 of final state tb
  Double_t fCosThetaWM;      // cos(theta_W-) in tb  frame
  Double_t fPhiWM;           // phi_W-        in tb  frame
  Double_t fQ2WM;            // q^2 of final state W-
  Double_t fCosThetaF1;      // cos(theta_f1) in t   frame
  Double_t fPhiF1;           // phi_f1        in t   frame
  Double_t fQ2HP;            // q^2 of final state H+
  Double_t fCosThetaT2;      // cos(theta_t)  in H+  frame
  Double_t fPhiT2;           // phi_t         in H+  frame
  Double_t fQ2T2;            // q^2 of final state t
  Double_t fCosThetaWP;      // cos(theta_W+) in H+  frame
  Double_t fPhiWP;           // phi_W+        in H+  frame
  Double_t fQ2WP;            // q^2 of final state W+
  Double_t fCosThetaF2;      // cos(theta_f2) in W+  frame
  Double_t fPhiF2;           // phi_f2        in W+  frame
  Double_t fZBoost;          // p_z(cm)      in lab frame
  Double_t fEcmIP;           // Ecm after B-strahlung

  ClassDef(HCHCSpringBuf, 1)  // HCHCSpring event buffer
};

//_______________________________________________________________________
// =====================
//  class HCHCSpring
// =====================
//-----------------------------------------------------------------------
class HCHCSpring : public JSFSpring {
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
public:
   HCHCSpring(const char   *name  = "HCHCSpring", 
              const char   *title = "HCHC Spring test",
                HCHCBases  *bases = 0);
   virtual ~HCHCSpring();

   virtual Bool_t Initialize(); // overload the base class method

   ClassDef(HCHCSpring, 1)  // HCHCSpring class
};

//_______________________________________________________________________
// =====================
//  class HCBoson
// =====================
//-----------------------------------------------------------------------

class HCBoson: public GENPDTEntry {
public:
   HCBoson(Double_t m    = 400., 
           Double_t tanb =   2.,
	   Double_t mt   = 175.);
   virtual ~HCBoson() {}

   Double_t GetTanBeta() { return fTanBeta; }

private:
   void     Initialize();
   Double_t GamToFF(Double_t m1, Double_t m2, Complex_t gl, Complex_t gr, Double_t color);

private:
   Double_t fTanBeta;    // tan(beta)
   Double_t fTopMass;    // mt

   ClassDef(HCBoson, 1)  // X boson class
};

//_______________________________________________________________________
// =====================
//  class Top Quark
// =====================
//-----------------------------------------------------------------------

class TopQuark: public GENPDTEntry {
public:
   TopQuark(Double_t m    = 175.);
   virtual ~TopQuark() {}

private:
   void     Initialize();
   Double_t GamToFV(Double_t m1, Double_t m2, Double_t gl, Double_t gr, Double_t vkm);

   ClassDef(TopQuark, 1)  // Top Quark class
};
#endif
