#include <iostream>
#include <iomanip>
#include <cmath>
#include <tuple>

#include "TApplication.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TH1D.h"

//#define __DEBUG__

//-----------------------------------------------------------------------------
//-------------------------------------
// Cross Section Calculator for
//      e+e- -> gamma H
// through electron Yukawa interaction
//-------------------------------------
// How to compile and run this
// $ make
// $ ./dsig [Ecm pole polp csmin csmax] # cntl-C to quit
// where
//    Ecm   = center of mass energy (250.) [GeV]
//    pole  = electron polarization (0.)
//    polp  = positron polarization (0.)
//    csmin = cos(theta)_min (photon acceptance limit: -1.)
//    csmax = cos(theta)_max (photon acceptance limit: +1.)
// with the numbers in the parentheses being default values.
// For instance, the following
// $ ./dsig 250.l 0.l 0.l -0.999l 0.999l
// calculates the cross section for -0.999l < cos(theta_photon) < +0.999l.
// Notice that all the input values are supposed to be of the quad precision.
//-----------------------------------------------------------------------------
//--
// Numerical constants (long double)
//--
constexpr auto kGeV2fb = 0.389379292e12l;   // GeV to fb
constexpr auto kPi     = acos(-1.l);
constexpr auto k4Pi    = 4*kPi;
constexpr auto k8Pi    = 8*kPi;
constexpr auto kAlpha  = 1/128.l;   // alpha(mz)
constexpr auto kSin2W  = 0.230l;
constexpr auto kSinW   = sqrt(kSin2W); 
constexpr auto kCos2W  = (1-kSinW)*(1+kSinW);
constexpr auto kCosW   = sqrt(kCos2W);
constexpr auto kGe     = sqrt(k4Pi*kAlpha);
constexpr auto kGw     = kGe/kSinW;
constexpr auto kGz     = kGw/kCosW;
constexpr auto kM_e    = 0.510998902e-3l; // [GeV]
constexpr auto kM_z    = 91.188l;         // [GeV]
constexpr auto kM_h    = 125.l;           // [GeV]
constexpr auto kVEV    = 2*kM_z/kGz;

//--
// Differential cross section: dsigma/dcos(theta)
//--
auto dsig(long double cs, long double rs)
{
   auto betab = [](long double M, long double m1, long double m2)
   {
	   auto x1 = pow(m1/M,2);
	   auto x2 = pow(m2/M,2);
	   
	   return x1*x2 != 0.l ? sqrt(1 - 2*(x1+x2) + pow(x1-x2,2)) :
                  (x1 == 0.l ? (1-m2/M)*(1+m2/M) : (1-m1/M)*(1+m1/M));
   };

   //--
   // Kinematics
   //--
   // initial state
   auto ebm   = rs/2.l; // beam energy
   auto pbm   = sqrt((ebm - kM_e)*(ebm + kM_e)); // beam momentum
   auto betae = pbm/ebm; // beta of the beam

   // final state
   auto bh  = betab(rs, 0.l, kM_h); // beta_bar of the final state
   auto pa  = ebm*bh; // absolute value of photon momentum

   // s, t, u, and propagators, Dt and Du
   auto s = rs*rs;
   auto Dti  = 2*ebm*pa*(1-betae*cs);
   auto rDti = sqrt(Dti);
   auto t    = (kM_e - rDti)*(kM_e + rDti);
   auto Dt   = -1/Dti; // t-channel electron propagator
   auto Dui  = 2*ebm*pa*(1+betae*cs);
   auto rDui = sqrt(Dui);
   auto u    = (kM_e - rDui)*(kM_e + rDui);
   auto Du   = -1/Dui; // u-channel electron propagator 

   
   // Amplitude squared
   //--
   auto amp2 = 2*pow(kGe*kM_e/kVEV,2)*Dt*Du
	       *(pow((rs-kM_h)*(rs+kM_h),2)*(Dt*t)*(Du*u) + 2*pow(kM_h*rs,2));

   //--
   // Polarized differential cross sections
   //--
   auto flux = (1/(2*s*betae));
   auto dph  = (bh/k8Pi)*(1.l/2);
   auto sgll  = flux*amp2*dph*kGeV2fb;
   auto sgrr  = sgll;
   auto sglr  = 0.l;
   auto sgrl  = 0.l;

   //--
   // That's it, folks!
   //--
   return std::tuple<long double,long double,long double,long double>{sgll,sglr,sgrl,sgrr};
}

//--
// main function (named as dsig)
//--
// argv[1] = Ecm [GeV]
//     [2] = pole
//     [3] = polp
//     [4] = cos(theta)_min
//     [5] = cos(theta)_max

int main(int argc, char *argv[])
{
   //--
   // Some lambdas for kinematics
   //--
   // cos(theta) as a function of pseudo rapidity
   auto csth = [](long double eta)
   {
	   auto x       = exp(2*eta);
	   auto cs      = (x - 1)/(x + 1);
	   auto dcsdeta = 4*x/(x+1)/(x+1); // Jacobian: cos->eta
	   return std::tuple<long double,long double>{cs,dcsdeta};
   };

   // pseudo rapidity as a function of cos(theta)
   auto prap = [](long double cs)
   {
	   return 0.5l*log((1+cs)/(1-cs));
   };

   //--
   // Parse command line arguments 
   //--
   auto rs    = 250.l; // Ecm [GeV]
   auto pole  = 0.0l;  // e- polarization
   auto polp  = 0.0l;  // e+ polarization
   auto csmin = -1.l;  // cos(theta)_min
   auto csmax = +1.l;  // cos(theta)_max
   switch (argc) {
      case 6:
         rs    = atof(argv[1]);
         pole  = atof(argv[2]);
         polp  = atof(argv[3]);
         csmin = atof(argv[4]);
         csmax = atof(argv[5]);
	 break;
      case 5:
         rs    = atof(argv[1]);
         pole  = atof(argv[2]);
         polp  = atof(argv[3]);
         csmin = atof(argv[4]);
	 break;
      case 4:
         rs    = atof(argv[1]);
         pole  = atof(argv[2]);
         polp  = atof(argv[3]);
	 break;
      case 3:
         rs    = atof(argv[1]);
         pole  = atof(argv[2]);
	 break;
      case 2:
         rs    = atof(argv[1]);
	 break;
      default:
	 break;
   }

   //--
   // Set eta boundary
   //--
#if 1
   auto etamin = -18.l;
   auto etamax = +18.l;
#else
   auto etamin = -15.l;
   auto etamax = +15.l;
#endif
   if (csmin != -1.l) etamin = prap(csmin);
   if (csmax != +1.l) etamax = prap(csmax);
   std::cerr << "etamin = " << etamin << " etamax = " << etamax << std::endl;
   std::cerr << "csmin = " << std::get<0>(csth(etamin)) 
             << " csmax = " << std::get<0>(csth(etamax)) << std::endl;
   auto neta   = 100;
   auto deta   = (etamax-etamin)/neta;

   //--
   // Loop over eta and integrate dsig using Simpson's formula
   //--
   auto wt  = 1.l;
   auto sum = 0.l;
   double xdata[neta+1], ydata[neta+1];
   for (int ieta=0; ieta<=neta; ieta++) {
      auto eta = etamin + ieta*deta;
      auto [cs, dcsdeta]  = csth(eta); // cos(theta) and dcos(theta)/deta
      auto [sgll, sglr, sgrl, sgrr] = dsig(cs, rs); // polarized cross sections
      auto val = ((1-pole)*(1-polp)*sgll+(1-pole)*(1+polp)*sglr
                 +(1+pole)*(1-polp)*sgrl+(1+pole)*(1+polp)*sgrr)/4;
      if (ieta == 0 || ieta == neta)   wt = 1.l/3;
      else if(ieta%2 == 0)             wt = 2.l/3;
      else                             wt = 4.l/3;
      sum += val*dcsdeta*deta*wt;
#ifdef __DEBUG__
      std::cerr << std::setprecision(30) << cs << " " << val << std::endl; 
#endif
      xdata[ieta] = cs;
      ydata[ieta] = val;
   }
   std::cerr << "sigtot = " << sum << "[fb]" << std::endl;

   //--
   // Plot dsig
   //--
   TApplication app("dsig", &argc, argv, 0, 0);

   TCanvas c1("c1","c1",800,600);
   TGraph  gr(neta+1,xdata,ydata);
   gr.SetTitle("");
   gr.SetLineColor(kRed);
   c1.SetLogy();
   c1.SetGridx();
   c1.SetGridy();
   gr.GetHistogram()->GetXaxis()->SetLimits(csmin,csmax);
   gr.GetHistogram()->GetXaxis()->SetTitle("cos#theta");
   gr.GetHistogram()->GetYaxis()->SetTitle("d#sigma/dcos#theta [fb]");
   gr.Draw("apc");

   app.Run();

   //--
   // That's it, folks!
   //--
   return 0;
}
