//*************************************************************************   
//* ---------------------                      
//* S4CALRealCluster Class
//* ---------------------   
//* (Description) 
//*     represent calorimeter cluster
//* (Requires)
//*     S4CALHit
//* (Provides)  
//*     S4CALRealCluster            : made object class
//* (Update Record)
//*     2004/09/03  S.Yamamoto  Original version.
//*     2004/11/15  S.Yamamoto  Add IsOK
//*************************************************************************

#include "JSFSteer.h"
#include "S4CALRealCluster.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4TPCPostHit.h"
#include "S4CALAddress.h"
#include "S4CALSmallClusterUtils.h"

#include "TVirtualPad.h"
#include "TPolyMarker3D.h"
#include <iostream>

//#define __DEBUG__

// temporary calibration factor
static Double_t calibEM = S4CALSmallCluster::fgCalibFactorForEM;
static Double_t calibHD = S4CALSmallCluster::fgCalibFactorForHD;
//static Double_t calibEM = S4VCluster::fgCalibFactorForEM;
//static Double_t calibHD = S4VCluster::fgCalibFactorForHD;

Int_t S4CALRealCluster::fRealClusterIDAll = -1;
Int_t S4CALRealCluster::fNEvt             = -1;

ClassImp(S4CALRealCluster)

//=====================================================================
//* constructor -------------------------------------------------------
S4CALRealCluster::S4CALRealCluster(S4HYBTrack      *trackp,
				   EClusterType_t   clustertype,
				   EParticleType_t  particletype)
  : fClusterR(0.), fClusterTheta(9999.), fClusterPhi(9999.),
    fTracks(1),
    fFracEM(100.), fClusterType(clustertype), fParticleType(particletype),
    fIsCalibrated(kFALSE),
    fIsPhoton(kFALSE), fIsElectron(kFALSE),
    fIsCHD(kFALSE), fIsNHD(kFALSE), fIsMuon(kFALSE), fIsNotDecided(kTRUE),
    fMinELayerID(1000), fMaxELayerID(-1000),
    fILPosition(0.,0.,0.), fMIPEnergy(0.)
{
  if(trackp) fTracks.Add(trackp);

  if(fNEvt != gJSF->GetEventNumber()){
    fRealClusterIDAll = 1;
    fNEvt = gJSF->GetEventNumber();
  }else{
    fRealClusterIDAll++;
  }

  fClusterID = fRealClusterIDAll;

}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALRealCluster::~S4CALRealCluster()
{
}

//=====================================================================
//* Add ---------------------------------------------------------------

void S4CALRealCluster::Add(const S4CALHit* hit)
{
  if(fIsCalibrated) fIsCalibrated = kFALSE;
  S4VCluster::Add(hit);
}

//=====================================================================
//* Add ---------------------------------------------------------------

void S4CALRealCluster::Add(S4CALSmallCluster *scl)
{
  if (fIsCalibrated) fIsCalibrated = kFALSE;
  std::vector<S4CALHit*> hvec = scl->GetCALHitsVector();
  for(Int_t k = 0; k < (Int_t) hvec.size(); k++)   S4VCluster::Add(hvec[k]);
}

//=====================================================================
//* AddMIPSmallCluser -------------------------------------------------

void S4CALRealCluster::AddMIPSmallCluster(S4CALSmallCluster* scl)
{

  //  static const Double_t bacalinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210);
  //  static const Double_t eccalinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
  static const Double_t bacalinR = S4CALAddress::GetBarrelInnerR();
  static const Double_t eccalinZ = S4CALAddress::GetBarrelHalfZ();

  Add(scl);

  if(scl->IsMIP()){
    fILPosition = scl->GetILPosition();
#if 0
    fMIPEnergy  = scl->GetEnergy();
#else
    TVector3 inpos  = (S4CALSmallClusterUtils::GetInTrack(scl->GetILTrackPtr(),bacalinR,eccalinZ))[0];
    Double_t length = (fILPosition - inpos).Mag(); // temporary
    fMIPEnergy  = length*20.e-3;  // temporary
#endif
  }
}

//=====================================================================
//* GetTrackPtr -------------------------------------------------------

S4HYBTrack *S4CALRealCluster::GetTrackPtr(Int_t i)
{
  return dynamic_cast<S4HYBTrack *>(fTracks[i]);
}

//=====================================================================
//* Update ------------------------------------------------------------

void S4CALRealCluster::Update()
{
  //   static const Int_t emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //   static const Int_t hdnlayers = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
   static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
   static const Int_t hdnlayers = S4CALAddress::GetHDNLayers();

   if(fDone) return;
   if(TObjArray::GetEntries() == 0) return;

   fCALHitsVector.clear();

   TIter nexthit(this);

   Int_t minlayerID  = 1000;
   Int_t maxlayerID  = -1000;
   Int_t minElayerID = 1000;
   Int_t maxElayerID = -1000;
   Double_t mlayer   = 0.;      // mean layer ID
   Double_t minE     = 1.e+15;
   Double_t maxE     = -1.e+15;
   Double_t layerE[emnlayers+hdnlayers];
   for(Int_t k = 0; k < (emnlayers+hdnlayers); k++){
     layerE[k] = 0.;
   }
   Int_t totalnhits  = 0;
   Double_t  ecl     = 0.;
   Double_t edepcl   = 0.;
   Double_t edepclem = 0.;
   Double_t  tcl     = 0.;
   TVector3  xcl     = TVector3(0.,0.,0.);
   S4CALHit *hitp    = 0;

   while ((hitp = dynamic_cast<S4CALHit *>(nexthit()))) {

     if(hitp->GetEnergyDeposit() < 1.e-6)  continue;

     totalnhits++;

     Double_t calib = 0.;
     hitp->IsEM() ? (calib = calibEM) : (calib = calibHD);

     Double_t energy = calib*(hitp->GetEnergyDeposit());

     edepcl += hitp->GetEnergyDeposit();
     if(hitp->IsEM()) edepclem += hitp->GetEnergyDeposit();
     ecl += energy;
     xcl += (hitp->GetCellPosition())*energy; 
     tcl += (hitp->GetTOF())*energy;


     Int_t layerID = S4CALAddress::GetLayerID(hitp->GetCellID(), hitp->IsEM());
     if(!hitp->IsEM()) layerID += emnlayers;
     
     if( layerID < minlayerID )  minlayerID = layerID;
     if( layerID > maxlayerID )  maxlayerID = layerID;
     
     layerE[layerID] += energy;
     mlayer += energy*layerID;

     fCALHitsVector.push_back(hitp);

   }  // end of while


   //==================================================== 
   // find max/min-Elayer which has maximum/minimum total energy deposit
   for(Int_t k = 0; k < (emnlayers+hdnlayers); k++){
     if(layerE[k] < minE && layerE[k] > 0.){
       minElayerID = k;
       minE = layerE[k];
     }
     if(layerE[k] > maxE){
       maxElayerID = k;
       maxE = layerE[k];
     }
   }
   //==================================================== 


   fEnergyDeposit = edepcl;
   if (edepcl <= 0.) return;
   fFracEM         = edepclem/edepcl;
   fTOF            = tcl/ecl;
   fPosition       = xcl;
   fPosition      *= 1./ecl;
   fClusterR       = fPosition.Mag();
   fClusterTheta   = fPosition.Theta();
   fClusterPhi     = fPosition.Phi();
   fNHits          = totalnhits;
   fMeanLayerID    = mlayer/ecl;
   fMinLayerID     = minlayerID;
   fMaxLayerID     = maxlayerID;
   fMinELayerID    = minElayerID;
   fMaxELayerID    = maxElayerID;

   fDone = kTRUE;
   
   MakeThrust();

   if(fIsCalibrated) fIsCalibrated = kFALSE;

   //--- cheating -----
   SetPIDMap();
   //------------------

}

//=====================================================================
//* Calibrate ---------------------------------------------------------  
                                          
void S4CALRealCluster::Calibrate()
{

  if(!fDone) Update();
  if(fIsCalibrated) return;

  fEnergy = 0.;

  Double_t edep        = GetEnergyDeposit();
  Double_t calibFactor = 0.;

  EParticleType_t ptype = GetParticleType();
  Int_t type = 0;

  if(ptype == kgElectron_p || ptype == kgPhoton_p) type = 1;
  else if(ptype == kgElectronPlusPhoton_p)         type = 2;
  else if(ptype == kgHdNeutral_p)                  type = 3;
  else if(ptype == kgHdCharged_p)                  type = 4;
  else if(ptype == kgMuon_p)                       type = 5;


  switch(type){

  case 1: // electron or photon
  case 2: // em-like particles
    //* def.
#if 0

#if 0
    if(edep < 0.006)  calibFactor = 1.497/TMath::Sqrt(edep) + 33.03;
    else              calibFactor = -2.204*TMath::Log(edep) + 41.21;
    //if(calibFactor < 48.) calibFactor = 48.;  // for high momentum particles
    if(calibFactor < 49.) calibFactor = 50.;  // for high momentum particles
#else
    //    calibFactor = 51.; before v1.22
    calibFactor = 36.5; // after v1.23
#endif
    fEnergy = calibFactor*edep;
    break;

#else
    //* default
    for(Int_t k = 0; k < (Int_t) fCALHitsVector.size(); k++){
      Bool_t isem = fCALHitsVector[k]->IsEM();
      calibFactor = isem ? calibEM : calibHD;
      fEnergy += calibFactor*(fCALHitsVector[k]->GetEnergyDeposit());
    }
    break;

#endif
  case 3: // neutral hadron

    for(Int_t k = 0; k < (Int_t) fCALHitsVector.size(); k++){
      Bool_t isem = fCALHitsVector[k]->IsEM();
      calibFactor = isem ? calibEM : calibHD;
      fEnergy += calibFactor*(fCALHitsVector[k]->GetEnergyDeposit());
    }
    break;

  case 4: // charged hadron

    //* def
#if 0
    for(Int_t k = 0; k < (Int_t) fCALHitsVector.size(); k++){
      Bool_t isem = fCALHitsVector[k]->IsEM();
      calibFactor = isem ?  42.8 : 39.5;
      fEnergy += calibFactor*(fCALHitsVector[k]->GetEnergyDeposit());
    }
    break;
#endif

  case 5: // muon

  default:

    //    std::cout << "#!#!# Calibration !!!! " << calibEM << " " << calibHD << std::endl;

    for(Int_t k = 0; k < (Int_t) fCALHitsVector.size(); k++){
      Bool_t isem = fCALHitsVector[k]->IsEM();
      calibFactor = isem ? calibEM : calibHD;
      fEnergy += calibFactor*(fCALHitsVector[k]->GetEnergyDeposit());
    }

  }

  fIsCalibrated = kTRUE;

#if 1
   fMomentum = fThrustVector*fEnergy;  // temporary momentum
#else
   fMomentum = fPosition*fEnergy/fPosition.Mag();
#endif

}

//=====================================================================
//* SetParticleType ---------------------------------------------------  
                                          
void S4CALRealCluster::SetParticleType(EParticleType_t type)
{

  if(fIsCalibrated) fIsCalibrated = kFALSE;

  fParticleType  = type;

  fIsElectron   = kFALSE;
  fIsPhoton     = kFALSE;
  fIsCHD        = kFALSE;
  fIsNHD        = kFALSE;
  fIsMuon       = kFALSE;
  fIsNotDecided = kFALSE;

  if(type == kgElectron_p)   fIsElectron   = kTRUE;
  if(type == kgPhoton_p)     fIsPhoton     = kTRUE;
  if(type == kgHdCharged_p)  fIsCHD        = kTRUE;
  if(type == kgHdNeutral_p)  fIsNHD        = kTRUE;
  if(type == kgMuon_p)       fIsMuon       = kTRUE;
  if(type == kgNotDecided_p) fIsNotDecided = kTRUE;

}

//=====================================================================
//* DebugPrint --------------------------------------------------------  

void S4CALRealCluster::DebugPrint(const Char_t *opt)
{
}
