//*************************************************************************
//* ---------------------
//* U4CDCHit Class
//* ---------------------
//* (Description)
//*	Class for a single CDC hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCHit		: TObject containing single hit data
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "U4CDCHit.h"
#include "U4CDCHitWire.h"
#include "U4CDCTimeToXRelation.h"


ClassImp(U4CDCCluster)
ClassImp(U4CDCHit)
	
U4CDCHit::U4CDCHit(U4CDCCluster *cluster) :
  	fDone(kFALSE), fClusterPtr(cluster),
  	fXdrift(0.), fDXdrift(0.), fVdrift(0.), fZ(0.), fDZ(0.),
	fWirePtr(0) {
#if 0
		std::cerr << " U4CDCHit::constructor "
			  << " HitPtr = " << (void *)this << std::endl;
#endif
	}

U4CDCHit::U4CDCHit(const U4CDCHit &h) : 
	fDone(h.fDone), fClusterPtr(h.fClusterPtr), 
	fXdrift(h.fXdrift), fDXdrift(h.fDXdrift), fVdrift(h.fVdrift),
        fZ(h.fZ), fDZ(h.fDZ),
	fWirePtr(h.fWirePtr) {}

U4CDCHit::~U4CDCHit() 
{	
#if 0
		std::cerr << " U4CDCHit::destructor "
			  << " HitPtr = " << (void *)this << std::endl;
#endif
}

Int_t	U4CDCHit::GetLayerNumber() const
{
	//std::cerr << "fWirePtr = " << (void *)fWirePtr << std::endl;
	if (fWirePtr) {
	        //std::cerr << "fWirePtr = " << fWirePtr->GetLayerNumber() << std::endl;
		return fWirePtr->GetLayerNumber();
	} else {
		Int_t layer, cell, wire;
		Double_t x, y;
		U4CDCHitWire::ChannelToWire(fClusterPtr,layer,cell,wire,x,y);
		return layer;
	}
}

Int_t	U4CDCHit::GetCellNumber () const
{
	if (fWirePtr) {
		return fWirePtr->GetCellNumber();
	} else {
		Int_t layer, cell, wire;
		Double_t x, y;
		U4CDCHitWire::ChannelToWire(fClusterPtr,layer,cell,wire,x,y);
		return cell;
	}
}

Int_t	U4CDCHit::GetWireNumber () const
{
	if (fWirePtr) {
		return fWirePtr->GetWireNumber();
	} else {
		Int_t layer, cell, wire;
		Double_t x, y;
		U4CDCHitWire::ChannelToWire(fClusterPtr,layer,cell,wire,x,y);
		return wire;
	}
}
  
Double_t U4CDCHit::Xwire() const
{
	if (fWirePtr) {
		return fWirePtr->X();
	} else {
		Int_t layer, cell, wire;
		Double_t x, y;
		U4CDCHitWire::ChannelToWire(fClusterPtr,layer,cell,wire,x,y);
		return x;
	}
}
  
Double_t U4CDCHit::Ywire() const
{
	if (fWirePtr) {
		return fWirePtr->Y();
	} else {
		Int_t layer, cell, wire;
		Double_t x, y;
		U4CDCHitWire::ChannelToWire(fClusterPtr,layer,cell,wire,x,y);
		return y;
	}
}
  

Bool_t U4CDCHit::IsSortable() const { return kTRUE; }

Int_t U4CDCHit::Compare(const TObject *obj) const
{
	// if (!fDone) Update();
	Double_t tim  = fClusterPtr->GetStartTime();
	Double_t timo = ((U4CDCHit *)obj)->fClusterPtr->GetStartTime();
   	if (tim < timo) return -1;
   	else if (tim > timo) return 1;
   	else return 0;
}

void U4CDCHit::DebugPrint(const Char_t *opt)
{
	if (!fDone) Update();
	std::cerr.setf(std::ios::right);
	std::cerr.setf(std::ios::fixed);
	std::cerr	
#if 1
	        << " pk = " << std::setw(5) << std::setprecision(1) << fClusterPtr->GetPeak()
#endif
		<< " (X,Y) = (" << std::setw(5) << std::setprecision(3) << fWirePtr->X()
		<< "," << std::setw(6) << std::setprecision(3) << fWirePtr->Y() << ")"
		<< " d = "  << std::setw(6) << std::setprecision(3) << fXdrift
		<< " dd = " << std::setw(6) << std::setprecision(4) << fDXdrift
		<< std::endl;
#if 1
	if (strstr(opt,"Detailed")) {
		std::cerr 	<< "    "
			<< " P_A = " << std::setw(6) << std::setprecision(1) 
				  	  << fClusterPtr->GetSum() << std::endl;
    	}
#endif
	std::cerr.unsetf(std::ios::right);
	std::cerr.unsetf(std::ios::fixed);
}

void U4CDCHit::Update()
{
	if (fDone) return;
	fDone = kTRUE;
	CalculatePosition();
}

void U4CDCHit::CalculatePosition()
{
	if (fWirePtr) {
#if 0 
		std::cerr << "fWirePtr =" << (void *)fWirePtr << std::endl;
		std::cerr << "fCalibPtr =" << (void *)fWirePtr->fCalibPtr << std::endl;
		std::cerr << "fClusterPtr =" << (void *)fClusterPtr << std::endl;
#endif
                fWirePtr->fCalibPtr->TimeToHitInfo(fClusterPtr,fXdrift,fDXdrift,fVdrift);
	} else {
		std::cerr 	<< "::::: Error in U4CDCHit::CalculatePosition"  << std::endl
			<< " This hit belong to no U4CDCHitWire object!" << std::endl
			<< " Append this to some U4CDCHitWire object!"   << std::endl;
	}
}

